from setuptools import setup, find_packages



setup(
	name="pyxtractors",
	version="0.0.5",
	packages=find_packages(),
	author="Alexander Smirnov",
	author_email="alexsm.w@yandex.ru",
	description="Tools to configure a mapping extraction policy"
)